print ("Tiny Foret by Midnoclose") --Tiny Forêt by Midnoclose
print ("Original tileset loading engine from https://love2d.org/wiki/Tutorial:Efficient_Tile-based_Scrolling")
print ("Tileset loading engine modified by Midnoclose")
print("Tileset by Midnoclose") --Tileset is 100% Open-Source, so you can do what ever you want with it!
print("Palette by DawnBringer https://github.com/geoffb/dawnbringer-palettes")

local map -- stores tiledata
local mapWidth, mapHeight -- width and height in tiles
 
local mapX, mapY -- view x,y in tiles. can be a fractional value like 3.25.
 
local tilesDisplayWidth, tilesDisplayHeight -- number of tiles to show
local zoomX, zoomY
 
local tilesetImage
local tileSize -- size of tiles in pixels
local tileQuads = {} -- parts of the tileset used for different tiles
local tilesetSprite
 
function love.load()
  setupMap()
  setupMapView()
  setupTileset()
  if true then
  love.draw()
  end
end
 
function setupMap()
  mapWidth = 200
  mapHeight = 200
 
  map = {}
  for x=1,mapWidth do
    map[x] = {}
    for y=1,mapHeight do
      map[x][y] = love.math.random(0,8)
    end
  end
end
 
function setupMapView()
  windowWidth = love.graphics.getWidth()
  windowHeight = love.graphics.getHeight()
  
  tileSize = 16
  
  mapX = 1
  mapY = 1
  tilesDisplayWidth = 26 -- Render distance width
  tilesDisplayHeight = 20 -- Render distance height
 
  zoomX = 32 / tileSize -- Tile Width
  zoomY = 32 / tileSize -- Tile Height
end
 
function setupTileset()
  tilesetImage = love.graphics.newImage( "foret16.png" )
  tilesetImage:setFilter("nearest") -- removed "linear" filter because it was showing peices of other sprites and making the image blur.
 
  -- Grass
  tileQuads[0] = love.graphics.newQuad(0 * tileSize, 0 * tileSize, tileSize, tileSize,
    tilesetImage:getWidth(), tilesetImage:getHeight())
  -- Dirt
  tileQuads[1] = love.graphics.newQuad(1 * tileSize, 0 * tileSize, tileSize, tileSize,
    tilesetImage:getWidth(), tilesetImage:getHeight())
  -- Wall Top
  tileQuads[2] = love.graphics.newQuad(2 * tileSize, 0 * tileSize, tileSize, tileSize,
    tilesetImage:getWidth(), tilesetImage:getHeight())
  -- Ice Wall
  tileQuads[3] = love.graphics.newQuad(3 * tileSize, 0 * tileSize, tileSize, tileSize,
    tilesetImage:getWidth(), tilesetImage:getHeight())
  -- Vine Wall
  tileQuads[4] = love.graphics.newQuad(4 * tileSize, 0 * tileSize, tileSize, tileSize,
    tilesetImage:getWidth(), tilesetImage:getHeight())
  -- Wood Wall
  tileQuads[5] = love.graphics.newQuad(5 * tileSize, 0 * tileSize, tileSize, tileSize,
    tilesetImage:getWidth(), tilesetImage:getHeight())
  -- Stone Wall
  tileQuads[6] = love.graphics.newQuad(0 * tileSize, 1 * tileSize, tileSize, tileSize,
    tilesetImage:getWidth(), tilesetImage:getHeight())
  -- Water Frame 1
  tileQuads[7] = love.graphics.newQuad(1 * tileSize, 1 * tileSize, tileSize, tileSize,
    tilesetImage:getWidth(), tilesetImage:getHeight())
  -- Water Frame 2
  tileQuads[8] = love.graphics.newQuad(2 * tileSize, 1 * tileSize, tileSize, tileSize,
    tilesetImage:getWidth(), tilesetImage:getHeight())

  tilesetBatch = love.graphics.newSpriteBatch(tilesetImage, tilesDisplayWidth * tilesDisplayHeight)
 
  updateTilesetBatch()
end
 
function updateTilesetBatch()
  tilesetBatch:clear()
  for x=0, tilesDisplayWidth-1 do
    for y=0, tilesDisplayHeight-1 do
      tilesetBatch:add(tileQuads[map[x+math.floor(mapX)][y+math.floor(mapY)]],
        x*tileSize, y*tileSize)
    end
  end
  tilesetBatch:flush()
end
 
-- central function for moving the map
function moveMap(dx, dy)
  oldMapX = mapX
  oldMapY = mapY
  mapX = math.max(math.min(mapX + dx, mapWidth - tilesDisplayWidth), 1)
  mapY = math.max(math.min(mapY + dy, mapHeight - tilesDisplayHeight), 1)
  -- only update if we actually moved
  if math.floor(mapX) ~= math.floor(oldMapX) or math.floor(mapY) ~= math.floor(oldMapY) then
    updateTilesetBatch()
  end
end
 
function love.update(dt)
  if love.keyboard.isDown("up", "w")  then
    moveMap(0, -0.3 * tileSize * dt)
  end
  if love.keyboard.isDown("down", "s")  then
    moveMap(0, 0.3 * tileSize * dt)
  end
  if love.keyboard.isDown("left","a")  then
    moveMap(-0.3 * tileSize * dt, 0)
  end
  if love.keyboard.isDown("right", "d")  then
    moveMap(0.3 * tileSize * dt, 0)
  end
end
 
function love.draw()
  love.graphics.draw(tilesetBatch,
    math.floor(-zoomX*(mapX%1)*tileSize), math.floor(-zoomY*(mapY%1)*tileSize),
    0, zoomX, zoomY)
  if debug == true then
  love.graphics.print("FPS: "..love.timer.getFPS(), 10, 20)
  end
end